@@ -1,20 +1,19 @@
 #include "Metis_utils.h"
 
-void ssc_metis_order(int n, int nnz,
-		     const int Tp [], const int Ti [],
+void ssc_metis_order(int n, const int Tp [], const int Ti [],
 		     idxtype* perm, idxtype* iperm)
 {
-    int i, j, ip, nnodes, num_flag = 0, options_flag = 0;
+    int  j, num_flag = 0, options_flag = 0;
     idxtype *xadj, *adj;
 
-    nnodes = 2 * (nnz - n);
     xadj = (idxtype *) R_alloc(n + 1, sizeof(idxtype));
-    adj = (idxtype *) R_alloc(nnodes, sizeof(idxtype));
+    adj = (idxtype *) R_alloc(2 * (Tp[n] - n), sizeof(idxtype));
 				/* temporarily use perm to store lengths */
-    for (j = 0; j < n; j++) perm[j] = 0;
+    memset(perm, 0, sizeof(idxtype) * n);
     for (j = 0; j < n; j++) {
-	for (ip = Tp[j]; ip < Tp[j+1]; ip++) {
-	    i = Ti[ip];
+	int ip, p2 = Tp[j+1];
+	for (ip = Tp[j]; ip < p2; ip++) {
+	    int i = Ti[ip];
 	    if (i != j) {
 		perm[i]++;
 		perm[j]++;
@@ -24,10 +23,11 @@ void ssc_metis_order(int n, int nnz,
     xadj[0] = 0;
     for (j = 0; j < n; j++) xadj[j+1] = xadj[j] + perm[j];
 				/* temporarily use perm to store pointers */
-    for (j = 0; j < n; j++) perm[j] = xadj[j];
+    Memcpy(perm, xadj, n);
     for (j = 0; j < n; j++) {
-	for (ip = Tp[j]; ip < Tp[j+1]; ip++) {
-	    i = Ti[ip];
+	int ip, p2 = Tp[j+1];
+	for (ip = Tp[j]; ip < p2; ip++) {
+	    int i = Ti[ip];
 	    if (i != j) {
 		adj[perm[i]] = j;
 		adj[perm[j]] = i;
