@@ -24,7 +24,8 @@ SEXP ssclme_coefGetsUnc(SEXP x, SEXP coef);
 SEXP ssclme_coefGets(SEXP x, SEXP coef);
 SEXP ssclme_EMsteps(SEXP x, SEXP nsteps, SEXP REMLp, SEXP verb);
 SEXP ssclme_fitted(SEXP x, SEXP facs, SEXP mmats);
-SEXP ssclme_variances(SEXP x, SEXP REML);
+SEXP ssclme_variances(SEXP x);
 SEXP ssclme_gradient(SEXP x, SEXP REMLp, SEXP Uncp);
+SEXP ssclme_collapse(SEXP x);
 
 #endif
